import type IDbModel from "@interfaces/database/IDbModel";
import MysqlService from "@services/mysql.service";
import {Logger} from "tslog";
import { v4 as uuidv4 } from 'uuid';

const DbHandler = new MysqlService.Handler('ModelService')
const logger = new Logger({name: 'ModelService'})

//SEC TODO validate blob
/**
 * Creates a new model with the provided data.
 *
 * @param {IDbModel} data - The data for the new model.
 *
 * @return {Promise<boolean>} - Indicates whether the model was created successfully.
 */
async function createModel(data: IDbModel): Promise<boolean> {
	logger.info(`Creating a new model... (${data.display_name})`)
	//TODO Validate IDbModel data
	try {
		await MysqlService.Model.insert(DbHandler, {
			id: uuidv4(),
			display_name: data.display_name,
			slug_name: data.slug_name,
			image_blob: data.image_blob,
			brand_id: data.brand_id,
			category_id: data.category_id,
			base_price: data.base_price,
			is_trending: data.is_trending
		})
		//TODO Return the new id
		logger.info('Success !')
		return true;
	} catch (error) {
		logger.error(`Error creating category: ${error}`);
		return false;
	}
}

/**
 * Updates a model in the database.
 *
 * @param {IDbModel} data - The model data to update.
 *
 * @return {Promise<boolean>} - A promise that resolves to a boolean indicating whether the update was successful or not.
 */
async function updateModel(data: IDbModel): Promise<boolean> {
	logger.info(`Updating model... (${data.slug_name})`);
	try {
		await MysqlService.Model.update(DbHandler, {
			display_name: data.display_name,
			slug_name: data.slug_name,
			image_blob: data.image_blob,
			brand_id: data.brand_id,
			category_id: data.category_id,
			base_price: data.base_price,
			is_trending: data.is_trending,
		});
		logger.info("Update Successful !");
		return true;
	} catch (error) {
		logger.error(`Error updating model: ${error}`);
		return false;
	}
}

/**
 * Deletes a model from the database.
 *
 * @param {string} modelSlug - The slug of the model to be deleted.
 * @return {Promise<boolean>} - A promise that resolves to true if the deletion is successful, else false.
 */
async function deleteModel(modelSlug: string): Promise<boolean> {
	if (!modelSlug) {
		logger.error("Model slug is missing");
		return false
	}
	logger.info(`Deleting model with ID: ${modelSlug}`);
	const doesExist = await MysqlService.Model.getBySlug(DbHandler, modelSlug);
	if (!doesExist || !doesExist.id) {
    logger.warn(`Model with slug ${modelSlug} not found`);
    return false;
  }
	try {
		await MysqlService.Model.delete(DbHandler, doesExist.id);
		logger.info("Deletion Successful !");
		return true;
	} catch (error) {
		logger.error(`Error deleting model: ${error}`);
		return false;
	}
}

const ModelService = {
	create: createModel,
	update: updateModel,
	delete: deleteModel,
	getBySlug: getBySlugModel,
	getAll: getAllModels,
	getById: getByIdModel,
	getByCategory: getByCategoryModel,
	getByBrand: getModelsByBrand,
}

export default ModelService;
